@@ -2,6 +2,7 @@
 from twisted.internet import defer, protocol
 from twisted.internet.task import LoopingCall
 from twisted.protocols import basic
+from twisted.python.failure import Failure
 from txamqp import spec
 from txamqp.codec import Codec
 from txamqp.connection import Header, Frame, Method, Body, Heartbeat
@@ -21,24 +22,35 @@ class GarbageException(Exception):
 # per connection
 class AMQChannel(object):
 
-    def __init__(self, id, outgoing):
+    def __init__(self, id, outgoing, client):
         self.id = id
         self.outgoing = outgoing
+        self.client = client
         self.incoming = TimeoutDeferredQueue()
         self.responses = TimeoutDeferredQueue()
 
         self.queue = None
 
+        self._closing = False
         self.closed = False
         self.reason = None
 
     def close(self, reason):
+        """Explicitely close a channel"""
+        self._closing = True
+        self.doClose(reason)
+        self._closing = False
+
+    def doClose(self, reason):
+        """Called when channel_close() is received"""
         if self.closed:
             return
         self.closed = True
         self.reason = reason
         self.incoming.close()
         self.responses.close()
+        if not self._closing:
+            self.client.channelFailed(self, Failure(reason))
 
     def dispatch(self, frame, work):
         payload = frame.payload
@@ -93,9 +105,12 @@ class AMQChannel(object):
         queue.put(header)
         for child in content.children:
             self.writeContent(klass, child, queue)
-        # should split up if content.body exceeds max frame size
         if size > 0:
-            queue.put(Frame(self.id, Body(content.body)))
+            maxChunkSize = self.client.MAX_LENGTH - 8
+            for i in xrange(0, len(content.body), maxChunkSize):
+                chunk = content.body[i:i + maxChunkSize]
+                queue.put(Frame(self.id, Body(chunk)))
+
 
 class FrameReceiver(protocol.Protocol, basic._PauseableMixin):
 
@@ -265,7 +280,7 @@ class AMQClient(FrameReceiver):
             try:
                 ch = self.channels[id]
             except KeyError:
-                ch = self.channelFactory(id, self.outgoing)
+                ch = self.channelFactory(id, self.outgoing, self)
                 self.channels[id] = ch
         finally:
             self.channelLock.release()
@@ -340,10 +355,12 @@ class AMQClient(FrameReceiver):
 
     @defer.inlineCallbacks
     def authenticate(self, username, password, mechanism='AMQPLAIN', locale='en_US'):
-        if self.check_0_8():
+        if mechanism == 'AMQPLAIN':
             response = {"LOGIN": username, "PASSWORD": password}
-        else:
+        elif mechanism == 'PLAIN':
             response = "\0" + username + "\0" + password
+        else:
+            raise ValueError('Unknown mechanism:'+mechanism)
 
         yield self.start(response, mechanism, locale)
 
@@ -356,7 +373,10 @@ class AMQClient(FrameReceiver):
         yield self.started.wait()
 
         channel0 = yield self.channel(0)
-        result = yield channel0.connection_open(self.vhost, insist=self.insist)
+        if self.check_0_8():
+            result = yield channel0.connection_open(self.vhost, insist=self.insist)
+        else:
+            result = yield channel0.connection_open(self.vhost)
         defer.returnValue(result)
 
     def sendHeartbeat(self):
@@ -375,3 +395,7 @@ class AMQClient(FrameReceiver):
             if self.checkHB.active():
                 self.checkHB.cancel()
         self.close(reason)
+
+    def channelFailed(self, channel, reason):
+        """Unexpected channel close"""
+        pass
